/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_POP_IMBALANCE_TEST_H
#define CUBELIB_POP_IMBALANCE_TEST_H

#include <string>
#include "PerformanceTest.h"


namespace mpianalysis
{
class POPParallelEfficiencyTest;

class POPImbalanceTest : public popcalculation::PerformanceTest
{
    friend class POPParallelEfficiencyTest;


private:
    cube::Metric* pop_comp;


protected:


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Load Balance Efficiency can be computed as follows: \n"
            "\tLoad Balance Efficiency =  computation time / maximum computation time)\n"
            "Thus it shows how big is a difference between average and maximal computation. ";
        std::string not_active_text =
            "Load Balance Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    POPImbalanceTest( cube::CubeProxy* );

    virtual
    ~POPImbalanceTest()
    {
    };

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );

    virtual
    double
    analyze( const cube::list_of_cnodes& cnodes,
             cube::LocationGroup*        _lg = nullptr ) const;

// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // POP_IMBALANCE_TEST_H
